!! Copyright INRIA. Contributors : Julien DIAZ, Abdelaaziz EZZIANI 
!! and Nicolas LEGOFF 
!! 
!! Julien.Diaz@inria.fr, Abdelaaziz.Ezziani@univ-pau.fr 
!! and Nicolas.Le_goff@inria.fr
!! 
!! This software is a computer program whose purpose is to
!! compute the analytical solution of problems of waves propagation in two 
!! layered media such as
!! - acoustic/acoustic
!! - acoustic/elastodynamic
!! - acoustic/porous
!! - porous/porous,
!! based on the Cagniard-de Hoop method.
!! 
!! This software is governed by the CeCILL license under French law and
!! abiding by the rules of distribution of free software.  You can  use, 
!! modify and/ or redistribute the software under the terms of the CeCILL
!! license as circulated by CEA, CNRS and INRIA at the following URL
!! "http://www.cecill.info". 
!! 
!! As a counterpart to the access to the source code and  rights to copy,
!! modify and redistribute granted by the license, users are provided only
!! with a limited warranty  and the software's author,  the holder of the
!! economic rights,  and the successive licensors  have only  limited
!! liability. 
!! 
!! In this respect, the user's attention is drawn to the risks associated
!! with loading,  using,  modifying and/or developing or reproducing the
!! software by the user in light of its specific status of free software,
!! that may mean  that it is complicated to manipulate,  and  that  also
!! therefore means  that it is reserved for developers  and  experienced
!! professionals having in-depth computer knowledge. Users are therefore
!! encouraged to load and test the software's suitability as regards their
!! requirements in conditions enabling the security of their systems and/or 
!! data to be ensured and,  more generally, to use and operate it in the 
!! same conditions as regards security. 
!! 
!! The fact that you are presently reading this means that you have had
!! knowledge of the CeCILL license and that you accept its terms.
!! ========================================================================

program Gar6more3D



  Use m_phys  
  Use m_num
  Use m_source
  Use m_sismo
  Use m_const
  implicit none

include 'mpif.h'
 
  integer I,J
  real*8 delta,A1inv(2,2),A2inv(2,2),Fstar(2),Mataux(2,2),P1inv(2,2)
  real*8 kb1,ks1,kf1,kb2,ks2,kf2,x,y,cost,sint

! MPI variables
  integer :: myrank, nproc
  integer :: ier
  integer, dimension(MPI_STATUS_SIZE) :: status

! dispatch variables
  integer, dimension(:), allocatable :: J_array_loc
  integer :: Nt_loc
  
! MPI initialization
  call MPI_INIT(ier)

  call MPI_COMM_SIZE(MPI_COMM_WORLD,nproc,ier)
  call MPI_COMM_RANK(MPI_COMM_WORLD,myrank,ier)


  pi=2.d0*dasin(1.D0)
  ima=(0,1)
  open(20,FILE="Gar6more3D.dat")
  if(myrank.eq.0) then
     write(6,*) 'Height of the source?'
  end if
  read(20,*) h
  if(myrank.eq.0) then
     write(6,*) h
     write(6,*) 'Frequency of the source?'
  end if
  read(20,*) f0
  if(myrank.eq.0) then
     write(6,*) f0
     write(6,*) 'Amplitude of the source?'
  end if
  read(20,*) amp
  if(myrank.eq.0) then
     write(6,*) amp
     write(6,*) 'delay of the source (f(t)=0 if t>2tdelay)?' 
     endif
  read(20,*) tdelay 
  if(myrank.eq.0) then
     write(6,*) tdelay 
     write(6,*) 'Height of the line of receivers?'
  end if
  read(20,*) Z
  if(myrank.eq.0) then
     write(6,*) Z
     write(6,*) 'Abscissa of the first receiver?'
  end if
  read(20,*) X1
  if(myrank.eq.0) then
     write(6,*) X1
     write(6,*) 'Ordinate of the first receiver?'
  end if
  read(20,*) Y1
  if(myrank.eq.0) then
     write(6,*) Y1
     write(6,*) 'Abscissa of the second receiver?'
  end if
  read(20,*) X2
  if(myrank.eq.0) then
     write(6,*) X2
     write(6,*) 'Ordinate of the second receiver?'
  end if
  read(20,*) Y2
  if(myrank.eq.0) then
     write(6,*) Y2
     write(6,*) 'How many receivers?'
  end if
  read(20,*) NX
  if(myrank.eq.0) then
     write(6,*) NX
  end if
  if (nx.ge.2) then
     dx=(X2-X1)/(Nx-1)
     dy=(Y2-Y1)/(Nx-1)
  else
     dx=0
     dy=0
  end if
  if(myrank.eq.0) then
     write(6,*) 'dx=',dx
     write(6,*) 'dy=',dy
     write(6,*) 'At which time shall the seismo begin?'
  end if
  read(20,*) T1
  if(myrank.eq.0) then
     write(6,*) T1
     write(6,*) 'At which time shall the seismo stop?'
  end if
  read(20,*) T2
  if(myrank.eq.0) then
     write(6,*) T2
     write(6,*) 'What is the time-step?'
  end if
  read(20,*) dt
  if(myrank.eq.0) then
     write(6,*) dt
  end if
  Nt=aint((T2-T1)/dt)+1

  if(myrank.eq.0) then
     write(6,*) 'How many intervals are required for the numerical computation of the time convolution?'
  end if
  read(20,*) Nint
  if(myrank.eq.0) then
     write(6,*) Nint
     write(6,*) 'How many intervals are required for the numerical computation of the space integration?'
  end if
  read(20,*) Nqint
  if(myrank.eq.0) then
     write(6,*) Nqint
  end if
  Nq02=max(10,Nqint/10)
  if(myrank.eq.0) then
     write(6,*) 'Is the medium heterogeneous (1) or not (0)?'
  end if
  read(20,*) heterogen
  if(myrank.eq.0) then
     write(6,*) heterogen
     write(6,*) 'Is the first medium acoustic (0) or porous (2)?'
  end if
  read(20,*) type_medium_1
  if(myrank.eq.0) then
     write(6,*) type_medium_1
  end if
  if (type_medium_1.eq.0) then
  if(myrank.eq.0) then
     write(6,*) 'mu and rho?'
  end if
     read(20,*) mu1,rho1
     if(myrank.eq.0) then
        write(6,*) mu1,rho1
     end if
     V1=dsqrt(mu1/rho1)
     if(myrank.eq.0) then         
        write(6,*) 'V1=', V1
     end if
  else
     if(myrank.eq.0) then
        write(6,*) 'kb (Frame bulk modulus), ks (Solid bulk modulus),&
          & kf (Fluid bulk modulus), mu (Frame shear modulus),&
          & rhof (Fluid density), rhos (Solid density), &
          &phi (Porosity), a(Tortuosity)?' 
     end if
     read(20,*) kb1,ks1,kf1,mu1,rhof1,rhos1,phi1,a1  
     if(myrank.eq.0) then
        write(6,*) kb1,ks1,kf1,mu1,rhof1,rhos1,phi1,a1  
     end if
        lambda1=kb1-2*mu1/3;
        rho1=phi1*rhof1+(1-phi1)*rhos1;
        rhow1=a1*rhof1/phi1;
        beta1=1-kb1/ks1;
        M1=1/(phi1/kf1+(beta1-phi1)/ks1);
        la1=lambda1+M1*(beta1-phi1)**2;
        R1=M1*phi1**2;
        ga1=M1*phi1*(beta1-phi1);
        S2=la1+2*mu1;
        AA1(1,1)=rho1 
        AA1(1,2)=rhof1
        AA1(2,1)=rhof1
        AA1(2,2)=rhow1;
        A1inv(1,1)=AA1(2,2)
        A1inv(1,2)=-AA1(1,2)
        A1inv(2,1)=-AA1(2,1)
        A1inv(2,2)=AA1(1,1)
        A1inv=A1inv/(AA1(1,1)*AA1(2,2)-AA1(1,2)**2)
        B1(1,1)=lambda1+2*mu1+M1*beta1**2 
        B1(1,2)=M1*beta1
        B1(2,1)=M1*beta1 
        B1(2,2)=M1;
        TT1=matmul(A1inv,B1);
        delta=(TT1(1,1)+TT1(2,2))**2+4*(TT1(1,2)*TT1(2,1)-TT1(1,1)*TT1(2,2))
        D1(1)=((TT1(1,1)+TT1(2,2))+sqrt(Delta))/2.
        D1(2)=((TT1(1,1)+TT1(2,2))-sqrt(Delta))/2.
        if (TT1(2,1).eq.0.d0) then
           if (D1(1).eq.TT1(1,1)) then
              P1(1,1)=1.;P1(2,1)=0.;
              if(TT1(1,2).eq.0.d0) then
                 P1(1,2)=0.;P1(2,2)=1.;
              else
                 P1(1,2)=-TT1(1,2)/(TT1(1,1)-D1(2));P1(2,2)=1.;
              end if
           else
              P1(1,2)=1.;P1(2,2)=0.
              if(TT1(1,2).eq.0.d0) then
                 P1(1,1)=0.;P1(2,1)=1.;
              else
                 P1(1,1)=-TT1(1,2)/(TT1(1,1)-D1(1));P1(2,1)=1.;
              end if
           end if
        else
           P1(1,1)=-(TT1(2,2)-D1(1))/TT1(2,1);P1(2,1)=1.
           P1(1,2)=-(TT1(2,2)-D1(2))/TT1(2,1);P1(2,2)=1.
        end if
        P1(:,1)=P1(:,1)/sqrt(sum(P1(:,1)**2))
        P1(:,2)=P1(:,2)/sqrt(sum(P1(:,2)**2))
        Vpsi1=sqrt(mu1*rhow1/(rho1*rhow1-rhof1**2))
        Vf1=sqrt(D1(1))
        Vs1=sqrt(D1(2))
     if(myrank.eq.0) then
        write(6,*) 'Vpsi1, Vf1, Vs1',Vpsi1,Vf1,Vs1
        write(6,*) 'P1'
        write(6,*) P1(1,1),P1(1,2)
        write(6,*) P1(2,1),P1(2,2)
     end if
  end if
  if (heterogen.eq.1) then
     if(myrank.eq.0) then
        write(6,*) 'Is the second medium acoustic (0) elastodynamic (1) or porous (2)?'
     end if
     read(20,*) type_medium_2
     if(myrank.eq.0) then
        write(6,*) type_medium_2
     end if
     SELECT CASE(type_medium_2)
     CASE(0)
     if(myrank.eq.0) then
        write(6,*) 'mu and rho?'
     end if
        read(20,*) mu2,rho2
     if(myrank.eq.0) then
        write(6,*) mu2,rho2
     end if
        V2=dsqrt(mu2/rho2)
     if(myrank.eq.0) then
        write(6,*) 'V2=', V2
     end if
     CASE(1)
     if(myrank.eq.0) then
        write(6,*) 'mu,lambda,rho'
     end if
        read(20,*) mu2,lambda2,rho2
     if(myrank.eq.0) then
        write(6,*) mu2,lambda2,rho2
     end if
        Vp2=dsqrt((lambda2+2*mu2)/rho2)
        Vs2=dsqrt(mu2/rho2)
     if(myrank.eq.0) then
        write(6,*) 'Vp2, Vs2',Vp2, Vs2
     end if
     CASE(2)
     if(myrank.eq.0) then
        write(6,*) ' kb (Frame bulk modulus), ks (Solid bulk modulus), kf (Fluid bulk modulus), mu (Frame shear modulus),&
         & rhof (Fluid density), rhos (Solid density), phi (Porosity), a(Tortuosity)?' 
     end if
        read(20,*) kb2,ks2,kf2,mu2,rhof2,rhos2,phi2,a2  
     if(myrank.eq.0) then
        write(6,*) kb2,ks2,kf2,mu2,rhof2,rhos2,phi2,a2  
     end if
        lambda2=kb2-2*mu2/3;
        rho2=phi2*rhof2+(1-phi2)*rhos2;
        rhow2=a2*rhof2/phi2;
        beta2=1-kb2/ks2;
        M2=1/(phi2/kf2+(beta2-phi2)/ks2);
        la2=lambda2+M2*(beta2-phi2)**2;
        R2=M2*phi2**2;
        ga2=M2*phi2*(beta2-phi2);
        S2=la2+2*mu2;
        AA2(1,1)=rho2 
        AA2(1,2)=rhof2
        AA2(2,1)=rhof2
        AA2(2,2)=rhow2;
        B2(1,1)=lambda2+2*mu2+M2*beta2**2 
        B2(1,2)=M2*beta2
        B2(2,1)=M2*beta2 
        B2(2,2)=M2;
        A2inv(1,1)=AA2(2,2)
        A2inv(1,2)=-AA2(1,2)
        A2inv(2,1)=-AA2(2,1)
        A2inv(2,2)=AA2(1,1)
        A2inv=A2inv/(AA2(1,1)*AA2(2,2)-AA2(1,2)**2)
        TT2=matmul(A2inv,B2);
        delta=(TT2(1,1)+TT2(2,2))**2+4*(TT2(1,2)*TT2(2,1)-TT2(1,1)*TT2(2,2))
        D2(1)=((TT2(1,1)+TT2(2,2))+sqrt(Delta))/2.
        D2(2)=((TT2(1,1)+TT2(2,2))-sqrt(Delta))/2.
        if (TT2(2,1).eq.0.d0) then
           if (D2(1).eq.TT2(1,1)) then
              P2(1,1)=1.;P2(2,1)=0.;
              if(TT2(1,2).eq.0.d0) then
                 P2(1,2)=0.;P2(2,2)=1.;
              else
                 P2(1,2)=-TT2(1,2)/(TT2(1,1)-D2(2));P2(2,2)=1.;
              end if
           else
              P2(1,2)=1.;P2(2,2)=0.
              if(TT2(1,2).eq.0.d0) then
                 P2(1,1)=0.;P2(2,1)=1.;
              else
                 P2(1,1)=-TT2(1,2)/(TT2(1,1)-D2(1));P2(2,1)=1.;
              end if
           end if
        else
           P2(1,1)=-(TT2(2,2)-D2(1))/TT2(2,1);P2(2,1)=1.
           P2(1,2)=-(TT2(2,2)-D2(2))/TT2(2,1);P2(2,2)=1.
        end if
        P2(:,1)=P2(:,1)/sqrt(sum(P2(:,1)**2))
        P2(:,2)=P2(:,2)/sqrt(sum(P2(:,2)**2))
        Vpsi2=sqrt(mu2*rhow2/(rho2*rhow2-rhof2**2))
        Vf2=sqrt(D2(1))
        Vs2=sqrt(D2(2))
     if(myrank.eq.0) then
        write(6,*) 'Vpsi2, Vf2, Vs2',Vpsi2,Vf2,Vs2
        write(6,*) 'P2'
        write(6,*) P2(1,1),P2(1,2)
        write(6,*) P2(2,1),P2(2,2)
     end if
     CASE DEFAULT
        write(6,*) 'Not implemented'
        stop
     end SELECT
  end if
! dispatching the time steps among the processors
  if (Nt < nproc) then
    write(6,*) 'Nt should be greater than nproc!'
    call MPI_ABORT(MPI_COMM_WORLD,-1,ier)
  endif
  
  allocate(J_array_loc(Nt))
  
  Nt_loc = 0
  do J=1,Nt
    if (modulo(J-1,nproc) == myrank) then
      Nt_loc = Nt_loc+1
      J_array_loc(Nt_loc) = J
    endif
  enddo
    if(myrank.eq.0) then
       write(6,*) 'Do you want a bulk source (1) or a pressure source (2) (only useful if first medium is poroelastic)?'
    end if
    read(20,*) type_source
    if(myrank.eq.0) then
       write(6,*) type_source
    end if
    if(myrank.eq.0) then
       write(6,*) 'Do you want open pore (1) or sealed pore (2) conditions (only useful for fluid/porous coupling)?'
   end if
   read(20,*) open
   if(myrank.eq.0) then
      write(6,*) open
   end if

  write(6,*) 'PROC',myrank,'got',Nt_loc,'of', Nt, 'time steps.'
  IF (heterogen.eq.0) then 
     SELECT CASE(type_medium_1)
     CASE(0)
        close(20)
        CALL acousacous(Nt_loc,J_array_loc,myrank)
     CASE(2)
        if (type_source==1) then
           Fstar(1)=1
           Fstar(2)=1
        else
           Fstar(1)=-M1*beta1
           Fstar(2)=-M1
        end if
        P1inv(1,1)=P1(2,2)
        P1inv(1,2)=-P1(1,2)
        P1inv(2,1)=-P1(2,1)
        P1inv(2,2)=P1(1,1)
        P1inv=P1inv/(P1(1,1)*P1(2,2)-P1(1,2)*P1(2,1))
        Mataux=matmul(P1inv,A1inv)
        Fstar=matmul(Mataux,Fstar);
        F1=Fstar(1);
        F2=Fstar(2);
     if(myrank.eq.0) then
        write(6,*) 'F1, F2', F1, F2
     end if
        close(20)
        CALL poroporo(Nt_loc,J_array_loc,myrank)
     CASE DEFAULT
        write(6,*) 'Not implemented'
        stop
     END SELECT
  else
     SELECT CASE(type_medium_1)
     CASE(0)
        close(20)
        SELECT CASE(type_medium_2)
        CASE(0)
           CALL acousacous(Nt_loc,J_array_loc,myrank)
        CASE(1)
        CALL acouselasto(Nt_loc,J_array_loc,myrank)
        CASE(2)
        CALL acousporo(Nt_loc,J_array_loc,myrank)
        CASE  DEFAULT
           write(6,*) 'Not implemented'
           stop
        END SELECT
     CASE(2)
        if (type_source==1) then
           Fstar(1)=1
           Fstar(2)=1
        else
           Fstar(1)=-M1*beta1
           Fstar(2)=-M1
        end if
        P1inv(1,1)=P1(2,2)
        P1inv(1,2)=-P1(1,2)
        P1inv(2,1)=-P1(2,1)
        P1inv(2,2)=P1(1,1)
        P1inv=P1inv/(P1(1,1)*P1(2,2)-P1(1,2)*P1(2,1))
        Mataux=matmul(P1inv,A1inv)
        Fstar=matmul(Mataux,Fstar);
        F1=Fstar(1);
        F2=Fstar(2);
     if(myrank.eq.0) then
        write(6,*) 'F1, F2', F1, F2
     endif
        close(20)
        CALL poroporo(Nt_loc,J_array_loc,myrank)
     CASE DEFAULT
        write(6,*) 'Not implemented'
        stop
     END SELECT
  END IF
  Do I=1,Nx
     x=X1+(I-1)*dx
     y=Y1+(I-1)*dy
     if (x**2+y**2.ne.0.) then
     cost=x/sqrt(x**2+y**2)
     sint=y/sqrt(x**2+y**2)
     Uy(i,:)=-sint*Ux(i,:)
     Ux(i,:)=-cost*Ux(i,:)
  end if
  END Do

! assembling the results
  if (myrank == 0) then
    do J = 1, Nt
       if (modulo(J-1,nproc) /= myrank) then
         call MPI_RECV(Ux(1,J),nx,MPI_DOUBLE_PRECISION,modulo(J-1,nproc),J,MPI_COMM_WORLD,status,ier)
         call MPI_RECV(Uy(1,J),nx,MPI_DOUBLE_PRECISION,modulo(J-1,nproc),J,MPI_COMM_WORLD,status,ier)
         call MPI_RECV(Uz(1,J),nx,MPI_DOUBLE_PRECISION,modulo(J-1,nproc),J,MPI_COMM_WORLD,status,ier)
	 if (type_medium_1.eq.0) then
	          call MPI_RECV(P(1,J),nx,MPI_DOUBLE_PRECISION,modulo(J-1,nproc),J,MPI_COMM_WORLD,status,ier)
         endif
       endif
    enddo
  else
    do J = 1, Nt
       if (modulo(J-1,nproc) == myrank) then
         call MPI_SEND(Ux(1,J),nx,MPI_DOUBLE_PRECISION,0,J,MPI_COMM_WORLD,ier)
	 call MPI_SEND(Uy(1,J),nx,MPI_DOUBLE_PRECISION,0,J,MPI_COMM_WORLD,ier)
	 call MPI_SEND(Uz(1,J),nx,MPI_DOUBLE_PRECISION,0,J,MPI_COMM_WORLD,ier)
 	 if (type_medium_1.eq.0) then
	    call MPI_SEND(P(1,J),nx,MPI_DOUBLE_PRECISION,0,J,MPI_COMM_WORLD,ier)
	 endif
       endif
    enddo
  endif

if (myrank == 0) then
  if(type_medium_1.eq.0) then
     open(20,FILE='Ux.dat')
     open(21,FILE='Uy.dat')
     open(22,FILE='Uz.dat')
     open(23,FILE='P.dat')
     do J=1,Nt
        write(20,*) T1+(j-1)*dt,(Ux(i,j),i=1,Nx)
        write(21,*) T1+(j-1)*dt,(Uy(i,j),i=1,Nx)
        write(22,*) T1+(j-1)*dt,(Uz(i,j),i=1,Nx)
        write(23,*) T1+(j-1)*dt,(P(i,j),i=1,Nx)
     END do
     close(20)
     close(21)
     close(22)
     close(23)
  else
     open(20,FILE='Ux.dat')
     open(21,FILE='Uy.dat')
     open(22,FILE='Uz.dat')
     do J=1,Nt
        write(20,*) T1+(j-1)*dt,(Ux(i,j),i=1,Nx)
        write(21,*) T1+(j-1)*dt,(Uy(i,j),i=1,Nx)
        write(22,*) T1+(j-1)*dt,(Uz(i,j),i=1,Nx)
     END do
     close(20)
     close(21)
     close(22)
  end if
endif

  call MPI_FINALIZE(ier)

end program Gar6more3D
