!! Copyright INRIA. Contributors : Julien DIAZ, Abdelaaziz EZZIANI 
!! and Nicolas LEGOFF 
!! 
!! Julien.Diaz@inria.fr, Abdelaaziz.Ezziani@univ-pau.fr 
!! and Nicolas.Le_goff@inria.fr
!! 
!! This software is a computer program whose purpose is to
!! compute the analytical solution of problems of waves propagation in two 
!! layered media such as
!! - acoustic/acoustic
!! - acoustic/elastodynamic
!! - acoustic/porous
!! - porous/porous,
!! based on the Cagniard-de Hoop method.
!! 
!! This software is governed by the CeCILL license under French law and
!! abiding by the rules of distribution of free software.  You can  use, 
!! modify and/ or redistribute the software under the terms of the CeCILL
!! license as circulated by CEA, CNRS and INRIA at the following URL
!! "http://www.cecill.info". 
!! 
!! As a counterpart to the access to the source code and  rights to copy,
!! modify and redistribute granted by the license, users are provided only
!! with a limited warranty  and the software's author,  the holder of the
!! economic rights,  and the successive licensors  have only  limited
!! liability. 
!! 
!! In this respect, the user's attention is drawn to the risks associated
!! with loading,  using,  modifying and/or developing or reproducing the
!! software by the user in light of its specific status of free software,
!! that may mean  that it is complicated to manipulate,  and  that  also
!! therefore means  that it is reserved for developers  and  experienced
!! professionals having in-depth computer knowledge. Users are therefore
!! encouraged to load and test the software's suitability as regards their
!! requirements in conditions enabling the security of their systems and/or 
!! data to be ensured and,  more generally, to use and operate it in the 
!! same conditions as regards security. 
!! 
!! The fact that you are presently reading this means that you have had
!! knowledge of the CeCILL license and that you accept its terms.
!! ========================================================================

SUBROUTINE poroporo(Nt_loc,J_array_loc,myrank)
  Use m_phys  
  Use m_num
  Use m_source
  Use m_sismo
  implicit none 

  integer :: Nt_loc,myrank
  integer, dimension(Nt) :: J_array_loc

!!! Computation of the incident wave
  allocate(Ux(1:nx,1:nt))
  allocate(Uy(1:nx,1:nt))
  allocate(Uz(1:nx,1:nt))
  Ux=0.d0
  Uy=0.d0
  Uz=0.d0
  if (heterogen.eq.1) then
     if (z.ge.0d0) then
        if (myrank.eq.0) then
           write(6,*) 'Computation of the incident Pf wave'
        end if
        CALL sub_incid_pp_f(Nt_loc,J_array_loc)
        if (myrank.eq.0) then
           write(6,*) 'Computation of the incident Ps wave'
        end if
        CALL sub_incid_pp_s(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the reflected PfPfwave'
        end if
        CALL sub_reflexff_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the reflected PfSwave'
        end if
        CALL sub_reflexfpsi_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the reflected PfPswave'
        end if
       CALL sub_reflexfs_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the reflected PsPfwave'
        end if
       CALL sub_reflexsf_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the reflected PsSwave'
        end if
        CALL sub_reflexspsi_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the reflected PsPswave'
        end if
       CALL sub_reflexss_pp(Nt_loc,J_array_loc)
     else
        if (myrank.eq.0) then
           write(6,*) 'Computation of the transmitted PfPfwave'
        end if
        CALL sub_transmitff_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the transmitted PfSwave'
        end if
        CALL sub_transmitfpsi_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the transmitted PfPswave'
        end if
        CALL sub_transmitfs_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the transmitted PsPfwave'
        end if
        CALL sub_transmitsf_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the transmitted PsSwave'
        end if
        CALL sub_transmitspsi_pp(Nt_loc,J_array_loc)
       if (myrank.eq.0) then
           write(6,*) 'Computation of the transmitted PsPswave'
        end if
        CALL sub_transmitss_pp(Nt_loc,J_array_loc)
     end if
  else
        if (myrank.eq.0) then
           write(6,*) 'Computation of the incident Pf wave'
        end if
        CALL sub_incid_pp_f(Nt_loc,J_array_loc)
        if (myrank.eq.0) then
           write(6,*) 'Computation of the incident Ps wave'
        end if
        CALL sub_incid_pp_s(Nt_loc,J_array_loc)
  end if
end SUBROUTINE poroporo
